#!/bin/bash

# SLES 9 only
# This script is written in bash (instead of ksh) because it invokes SLES scripts

heartbeat_msg="Cannot find heartbeat in /etc/ha.d/resource.d.\
               Please install the heartbeat RPMs required for high-availability.\
               Installation will not continue."
notify_msg="Warning: The installed version of util-linux does not support\
            notification of failures on IP addresses for NLM lock recovery. SLES9 \
            clients that use IP addresses to mount NFS servers will not be able to \
            reclaim locks on failures."
grace_msg="Warning: The running kernel does not support dynamic grace period for \
           NFS locking. On single node failure, all NFS locks will be released \
           cluster-wide and reclaimed by clients."

install_script=/usr/lpp/mmfs/samples/nfscluster/install_ha-nfs.sh

# Exit if status is not 0
statusCheck() {
    status=$?
    if [ $status -ne 0 ]; then
        echo "Installation failed. See $LOGFILE for details."
        exit $status
    fi
}

die() {
    echo $*
    echo "Installation failed."
    exit 0
}

logit() {
    echo "`date`: $*" >> $LOGFILE 2>&1
    $* >> $LOGFILE 2>&1
}

usage() {
    echo "Usage: $0 [-a] nfsdefs nodes.list"
    echo "$@"
    exit 1
}

config() {
    # Check for SUSE distribution
    dist=$(cat /etc/issue | grep "SUSE LINUX Enterprise Server 9")
    if [ -z "$dist" ]; then
	echo "Install will work only for SUSE LINUX Enterprise Server 9"
	return 1
    fi

    # Check the GPFS is not running
    gpfs=$(mmfsadm dump version 2> /dev/null | grep "Current daemon version")
    if [ -n "$gpfs" ]; then
	echo "GPFS must be stopped before installation."
	return 1
    fi

    # Ensure programs required for HA are available
    if [ ! -e /etc/ha.d/resource.d/IPaddr ]; then
	echo $heartbeat_msg
        return 1
    fi

    # Ensure all public interfaces used for NFS are static (no DHCP)
    gpfsip=$(myGPFSIP)
    nfsIPs=$(getNfsIPs $gpfsip)
    for ip in $nfsIPs; do
        # Find its ifcfg file
        ifcfg=$(grep -wl $ip $IFPATH/ifcfg-eth* 2> /dev/null)
	if [ -z "$ifcfg" ]; then
            echo "Cannot find configuration file for $ip in $IFPATH"
            echo "$ip needs to be a static IP address defined in $IFPATH. DHCP is not permitted as an NFS address."
            return 1
        fi
        bootproto=$(grep ^BOOTPROTO $ifcfg | awk -F\' '{print $2}')
        if [ "$bootproto" != "static" ]; then
            echo "$ip needs to be a static IP address in $ifcfg. DHCP is not permitted as an NFS address."
            return 1
        fi
    done

    release=$(rpm --qf "%{RELEASE}\n" -q util-linux)
    if [ "$release" != "72.38" ]; then
        echo $notify_msg
    fi

    checkStatus nfsd
    if [ $? -ne 0 ]; then
        logit nfsService start
    fi
    if [ ! -f /proc/fs/nfsd/grace ]; then
        echo $grace_msg
    fi

    # Copy files from samples
    [ -d /var/mmfs/etc ] && cp -a /var/mmfs/etc /var/mmfs/etc.bak
    cp -dp /usr/lpp/mmfs/samples/nfscluster/* /var/mmfs/etc

    logit cp -dp /usr/lpp/mmfs/samples/nfscluster/init.d.gpfs /etc/init.d/gpfs &&
    logit insserv gpfs
    # Undo GPFS autoload from inittab
    sed -i 's/^gpfs:\(.*\)/# gpfs:\1	# Disabled by HA-NFS/g' /etc/inittab

    # Set default start mode of interfaces to be used for HA-NFS to "manual"
    # This is required so clients do not detect active connections on failover
    # or failback before GPFS is active
    for ip in $nfsIPs; do
        # Find its ifcfg file
        ifcfg=$(grep -wl $ip $IFPATH/ifcfg-eth* 2> /dev/null)
	[ -z "$ifcfg" ] && continue
        iface=$(echo $ifcfg | sed 's/ifcfg-//g')
        if grep -q "^STARTMODE=\'onboot\'" $ifcfg; then
            # SuSE
            sed -i "s/^STARTMODE='onboot'/STARTMODE='manual'	# added for HA-NFS/g" $ifcfg
            echo "Installation has changed the start mode of interface $iface to manual."
        elif grep -q "^ONBOOT=yes" $ifcfg; then
            # RedHat
            sed -i "s/^ONBOOT=yes/ONBOOT=no	# added for HA-NFS/g" $ifcfg
            echo "Installation has changed the start mode of interface $iface to manual."
        fi
    done

    # Stop NFS server so mmstartup can work correctly
    logit nfsService stop
    logit nfsmonitor -e
    checkStatus nfsd
    if [ $? -eq 0 ]; then
	echo "Cannot stop NFS server. Reboot required to complete installation."
    fi
}

install() {
    nodeList=$(echo $gpfsIPs | sed 's/ /,/g')

    echo -n "Synchronizing GPFS files to all nodes in the cluster "
    # FIX: use rdist?
    for ip in $gpfsIPs; do
	logit $GPFS_rcpPath $defs $ip:/var/mmfs/etc
	logit $GPFS_rcpPath $nodes $ip:$NODELIST
    done
    rc_status -v
    statusCheck

    echo -n "Installing HA-NFS service on all nodes in the cluster "
    logit mmdshcmdRC $nodeList $install_script -c
    rc_status -v
    statusCheck
}

chconfig() {
    options=""

    # After creating the GPFS cluster turn on the GPFS HA-NFS option.
    options="enablenfscluster=yes"

    # Autoload GPFS on boot.
    if [ "$AUTOSTART" == "1" ]; then 
	options="$options,autoload=yes"
    else
	options="$options,autoload=no"
    fi

    # For better read performance of large files set the following option.
    options="$options,nfsprefetchstrategy=1"

    # Current lease time and wait time are 35+35 seconds.
    # set leaserecoverywait to no less than 10 second.
    options="$options,leaserecoverywait=10"

    # set leaseDuration to no less than 20 second.
    options="$options,leaseDuration=20"

    echo -n "Configuring GPFS cluster for HA-NFS "
    logit mmchconfig "$options"
    rc_status -v
}

touch /var/mmfs/etc/nfsdefs # workaround to get nfsfuncs to work
funcs=/usr/lpp/mmfs/samples/nfscluster/nfsfuncs
[ -f $funcs ] || die "Cannot find $funcs"
. $funcs

. /etc/rc.status
rc_reset

LOGFILE=/tmp/ha-nfs-install.log
> $LOGFILE

allflag=""
while getopts acv OPT; do
    case $OPT in
	a)
	    [ -n "$allflag" ] && usage "specify '$OPT' option only once"
	    allflag=yes
	    ;;
	c)
	    config
	    exit $?
	    ;;
	*)
	    usage "$OPTARG is not valid"
	    ;;
    esac
done

shift $(($OPTIND-1))
[ $# -eq 2 ] || usage

defs=$1; nodes=$2

[ -f $defs ] || die "Cannot find $defs. See /usr/lpp/mmfs/samples/nfscluster/README for installation instructions"
[ -f $nodes ] || die "Cannot find $nodes. See /usr/lpp/mmfs/samples/nfscluster/README for installation instructions"
. $defs

# Reset log file that might be overwritten by $defs
LOGFILE=/tmp/ha-nfs-install.log

if [ -n "$GPFS_RSH" ]; then
    GPFS_rshPath=$(which $GPFS_RSH)
else
    GPFS_rshPath=$(which rsh)
fi
if [ -n "$GPFS_RCP" ]; then
    GPFS_rcpPath=$(which $GPFS_RCP)
else
    GPFS_rcpPath=$(which rcp)
fi

# Check that mandatory parameters have been defined
[ -n "$SHARED_ROOT" ] || die "The system directory (SHARED_ROOT) for HA-NFS not defined in $defs"
[ -n "$NODELIST" ] || die "The list of GPFS nodes (NODELIST) for HA-NFS not defined in $defs"

if [ -z "$allflag" ]; then
    config
else
    # Run configuration on all GPFS nodes
    temp=$NODELIST
    NODELIST=$nodes 		# used by getAllGPFSIPs
    gpfsIPs=$(getAllGPFSIPs)
    NODELIST=$temp

    install $gpfsIPs
    chconfig
fi

RC=$?
if [ $RC -eq 0 ]; then
    echo -n "Installation is complete. "
else
    echo -n "Installation failed. "
fi
echo "See $LOGFILE for details."
exit $RC
